from setuptools import setup

setup(name='silver_captcha',
      version='1.0',
      description='Generates captcha',
      url='http://github.com/storborg/funniest',
      author='silvermoon',
      author_email='faith.frontiers@gmail.com',
      license='MIT',
      packages=['silver_captcha'],
      zip_safe=False)